@@ -214,3 +214,29 @@ if(!isGeneric("kStepEstimator.start")){
     setGeneric("kStepEstimator.start",
                 function(start,...) standardGeneric("kStepEstimator.start"))
 }
+if(!isGeneric("radius")){
+    setGeneric("radius", function(object) standardGeneric("radius"))
+}
+
+if(!isGeneric("samplesize<-")){
+    setGeneric("samplesize<-",
+        function(object, value) standardGeneric("samplesize<-"))
+}
+if(!isGeneric("getRiskFctBV")){
+    setGeneric("getRiskFctBV", function(risk, biastype) standardGeneric("getRiskFctBV"))
+}
+
+if(!isGeneric("moveL2Fam2RefParam")){
+    setGeneric("moveL2Fam2RefParam", function(L2Fam, ...)
+                standardGeneric("moveL2Fam2RefParam"))
+}
+
+if(!isGeneric("moveICBackFromRefParam")){
+    setGeneric("moveICBackFromRefParam", function(IC, L2Fam, ...)
+               standardGeneric("moveICBackFromRefParam"))
+}
+
+if(!isGeneric("rescaleFunction")){
+    setGeneric("rescaleFunction", function(L2Fam, ...)
+               standardGeneric("rescaleFunction"))
+}
