@@ -2,9 +2,6 @@
 ## k-step estimator
 ###############################################################################
 
-### helper functions from distrMod:
-.isUnitMatrix <- distrMod:::.isUnitMatrix
-.deleteDim <- distrMod:::.deleteDim
 
 ### no dispatch on top layer -> keep product structure of dependence
 kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
@@ -13,7 +10,10 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
                            IC.UpdateInKer = getRobAStBaseOption("IC.UpdateInKer"),
                            withICList = getRobAStBaseOption("withICList"),
                            withPICList = getRobAStBaseOption("withPICList"),
-                           na.rm = TRUE, startArgList = NULL, ...){
+                           na.rm = TRUE, startArgList = NULL, ...,
+                           withLogScale = TRUE, withEvalAsVar = TRUE){
+
+        if(missing(IC.UpdateInKer)) IC.UpdateInKer <- NULL
 ## save call
         es.call <- match.call()
         es.call[[1]] <- as.name("kStepEstimator")
@@ -68,13 +68,18 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
            start <- L2Fam@startPar
 
 ### use dispatch here  (dispatch only on start)
-        a.var <- if( is(start, "Estimate")) asvar(start) else NULL
+        #a.var <- if( is(start, "Estimate")) asvar(start) else NULL
         IC.UpdateInKer.0 <- if(is(start,"ALEstimate")) start@pIC else NULL
+        force(startArgList)
         start.val <- kStepEstimator.start(start, x=x0, nrvalues = k,
                          na.rm = na.rm, L2Fam = L2Fam,
                          startList = startArgList)
 
-
+### use Logtransform here in scale models
+        sclname <- ""
+        if(is(L2Fam, "L2ScaleUnion")) sclname <- scalename(L2Fam)
+        logtrf <- is(L2Fam, "L2ScaleUnion") &
+                     withLogScale & sclname %in% names(start.val)
 ### a starting value in k-space
         u.theta <- start.val
         theta <- if(is(start.val,"Estimate")) estimate(start.val)
@@ -88,16 +93,20 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
         pICList <- if(withPICList) vector("list", steps) else NULL
         ICList  <- if(withICList)  vector("list", steps) else NULL
 
+        cvar.fct <- function(L2, IC, dim, dimn =NULL){
+                if(is.null(dimn)){
+                   return(matrix(E(L2, IC %*% t(IC)),dim,dim))
+                }else{
+                   return(matrix(E(L2, IC %*% t(IC)),dim,dim, dimnames = dimn))
+                }
+        }
+
         ### update - function
         updateStep <- function(u.theta, theta, IC, L2Fam, Param,
                                withModif = TRUE, with.u.var = FALSE){
 
                 IC.c <- as(diag(p) %*% IC@Curve, "EuclRandVariable")
 
-
-                theta <- theta + rowMeans(evalRandVar(IC.c, x0),
-                                          na.rm = na.rm )
-
 #                print(theta)
                 tf <- trafo(L2Fam, Param)
                 Dtau <- tf$mat
@@ -125,9 +134,29 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
                                     "EuclRandVariable") else NULL
                      }
                      IC.tot <- IC.tot1 + IC.tot2
-                     u.theta <- u.theta + rowMeans(evalRandVar(IC.tot, x0),
-                                                   na.rm = na.rm)
+                     correct <- rowMeans(evalRandVar(IC.tot, x0), na.rm = na.rm)
+                     iM <- is.matrix(u.theta)
+                     names(correct) <- if(iM) rownames(u.theta) else names(u.theta)
+                     if(logtrf){
+                        scl <- if(iM) u.theta[sclname,1] else u.theta[sclname]
+                        u.theta <- u.theta + correct
+                        if(iM) u.theta[sclname,1] <- scl * exp(correct[sclname]/scl) else
+                               u.theta[sclname] <- scl * exp(correct[sclname]/scl)
+                     }else u.theta <- u.theta + correct
+
+                     theta <- (tf$fct(u.theta))$fval
                 }else{
+                     correct <- rowMeans(evalRandVar(IC.c, x0), na.rm = na.rm )
+                     iM <- is.matrix(theta)
+                     names(correct) <- if(iM) rownames(theta) else names(theta)
+                     if(logtrf){
+                        scl <- if(iM) theta[sclname,1] else theta[sclname]
+                        theta <- theta + correct
+                        if(iM) theta[sclname,1] <- scl * exp(correct[sclname]/scl) else
+                               theta[sclname] <- scl * exp(correct[sclname]/scl)
+                     }else{
+                        theta <- theta + correct
+                     }
                      IC.tot <- IC.c
                      u.theta <- theta
                 }
@@ -135,18 +164,29 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
                 var0 <- u.var <- NULL
                 if(with.u.var){
                    cnms <-  if(is.null(names(u.theta))) colnames(Dtau) else names(u.theta)
-                   if(!is.null(IC.tot.0))
-                      u.var <- matrix(E(L2Fam, IC.tot.0 %*% t(IC.tot.0)),
-                                  k,k, dimnames = list(cnms,cnms))
-                   if(!var.to.be.c)
-                       var0 <- matrix(E(L2Fam, IC.c %*% t(IC.c)),p,p)
+                   if(!is.null(IC.tot.0)){
+                      u.var <- substitute(do.call(cfct, args = list(L2F0, IC0,
+                                   dim0, dimn0)), list(cfct = cvar.fct,
+                                   L2F0 = L2Fam, IC0 = IC.tot.0, dim0 = k,
+                                   dimn0 = list(cnms,cnms)))
+                      if(withEvalAsVar) u.var <- eval(u.var)
+                     #         matrix(E(L2Fam, IC.tot.0 %*% t(IC.tot.0)),
+                     #             k,k, dimnames = list(cnms,cnms))
+                   }
+                   if(!var.to.be.c){
+                      var0 <- substitute(do.call(cfct, args = list(L2F0, IC0,
+                                   dim0, dimn0)), list(cfct = cvar.fct,
+                                   L2F0 = L2Fam, IC0 = IC.c, dim0 = p))
+                      if(withEvalAsVar) var0 <- eval(var0)
+                   }
                 }
 
                 if(withModif){
                    main(Param)[] <- .deleteDim(u.theta[idx])
                    if (lnx) nuisance(Param)[] <- .deleteDim(u.theta[nuis.idx])
 #                   print(L2Fam)
-                   L2Fam <- modifyModel(L2Fam, Param)
+                   L2Fam <- modifyModel(L2Fam, Param,
+                               .withL2derivDistr = L2Fam@.withEvalL2derivDistr)
 #                   print(L2Fam)
                    IC <- modifyIC(IC)(L2Fam, IC)
 #                   print(IC)
@@ -204,10 +244,10 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
               Param <- upd$Param
               tf <- trafo(L2Fam, Param)
               Infos <- rbind(Infos, c("kStepEstimator",
-                  "computation of IC, trafo, asvar and asbias via useLast = TRUE"))
+               "computation of IC, trafo, asvar and asbias via useLast = TRUE"))
            }else{
               Infos <- rbind(Infos, c("kStepEstimator",
-                       "computation of IC, trafo, asvar and asbias via useLast = FALSE"))
+               "computation of IC, trafo, asvar and asbias via useLast = FALSE"))
            }
         }else{
            if(steps > 1)
@@ -233,7 +273,7 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
 #        print(IC@Risks$asCov)
 #        print(Risks(IC)$asCov)
 
-        if(! distrMod:::.isUnitMatrix(trafo(L2Fam)))
+        if(! .isUnitMatrix(trafo(L2Fam)))
              Infos <- rbind(Infos, c("kStepEstimator",
                             paste("computation of IC",
                                    ifelse(withUpdateInKer,"with","without") ,
@@ -260,7 +300,8 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
         }else{
                 if(is(IC, "HampIC")){
                     r <- neighborRadius(IC)
-                    asBias <- r*getRiskIC(IC, risk = asBias(), neighbor = neighbor(IC))$asBias$value
+                    asBias <- r*getRiskIC(IC, risk = asBias(),
+                                          neighbor = neighbor(IC))$asBias$value
                 }else{
                     asBias <- NULL
                 }
@@ -279,20 +320,18 @@ kStepEstimator <- function(x, IC, start = NULL, steps = 1L,
           asVar <- asVar[idx,idx,drop=FALSE]
 #          print(asVar)
           names(theta) <- nms.theta.idx
-          dimnames(asVar) <- list(nms.theta.idx,nms.theta.idx)
+          dimnames(asVar) <- list(nms.theta.idx, nms.theta.idx)
         }
 
         return(new("kStepEstimate", estimate.call = es.call,
-                       name = paste(steps, "-step estimate", sep = ""),
-                       estimate = theta, samplesize = nrow(x0), asvar = asVar,
-                       trafo = tf, fixed = fixed,
-                       nuis.idx = nuis.idx, untransformed.estimate = u.theta,
-                       completecases = completecases,
-                       untransformed.asvar = u.var,
-                       asbias = asBias, pIC = IC, steps = steps, Infos = Infos,
-                       start = start, startval = start.val, ustartval = u.start.val,
-                       ksteps = ksteps, uksteps = uksteps,
-                       pICList = pICList, ICList = ICList))
+                name = paste(steps, "-step estimate", sep = ""),
+                estimate = theta, samplesize = nrow(x0), asvar = asVar,
+                trafo = tf, fixed = fixed, nuis.idx = nuis.idx,
+                untransformed.estimate = u.theta, completecases = completecases,
+                untransformed.asvar = u.var, asbias = asBias, pIC = IC,
+                steps = steps, Infos = Infos, start = start,
+                startval = start.val, ustartval = u.start.val, ksteps = ksteps,
+                uksteps = uksteps, pICList = pICList, ICList = ICList))
 }
 #  (est1.NS <- kStepEstimator(x, IC2.NS, est0, steps = 1))
 
